<?php

/**
 * @file
 *
 * Contains the sort handler that will sort the output
 * of PostgreSQL's ts_ranking() or ts_ranking_cd() function
 * making it possible to ranking results based on
 * their keyword score.
 *
 * Again, the used FTS filters are used as the base to
 * build the query on. This is needed since
 * ts_rank() requires both a ts_query input (which is
 * generated by our filter) and a FTS index (which is
 * also delivered by our filter).
 */

class postgresql_fts_views_handler_ranking_sort extends views_handler_sort {

  /**
   * Inform the system about the various option that we
   * set for this PostgreSQL sort handler, appending to
   * the options set by the parent class.
   */
  function option_definition() {
    $options = parent::option_definition();

    // Add the different options so they can be saved afterwards.
    $options['filter'] = array('default' => 0);
    $options['cover_density'] = array('default' => 0);
    $options['normalization'] = array('default' => 0);

    return $options;
  }


  /**
   * Create the actual options form, appending to
   * the options set by the parent class.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    // Which FTS filter should be used to rank with?
    $form['filter'] = array(
      '#type' => 'select',
      '#title' => t('Ranking filter'),
      '#description' => t('Choose which PostgreSQL FTS filter should be used for the ranking.'),
      '#options' => $this->_populate_filter_dropdown(),
      '#default_value' => $this->options['filter']);

    // Should we use normal ranking or ranking based on cover density?
    $form['cover_density'] = array(
      '#type' => 'checkbox',
      '#title' => t('Rank using cover density'),
      '#description' => t('Choose whether to rank normally (unchecked) or to rank by cover density (checked - take into account how far found keywords are positioned from one another).'),
      '#default_value' => $this->options['cover_density']);

    $normalization_options = array(
      0 => 'Ignore document length',
      1 => 'Divide rank by logarithm of document length',
      2 => 'Divide rank by simple document length',
      4 => 'Divide rank by mean harmonic distance between extents',
      8 => 'Divide rank by number of unique words',
      16 => 'Divide rank by logarithm of number of unique words',
      32 => 'Divide rank by itself');

    // Choose the normalization for the ts_rank()/ts_rank_cd() function.
    $form['normalization'] = array(
      '#type' => 'select',
      '#title' => t('Ranking normalization'),
      '#description' => t('Decide how to handle document length when ranking results. A short document which matches on four or five tokens has a different relevance than a three times as long document which matches on the same amount of tokens. The shorter one is probably more relevant then the longer one. The choice above will influence the calculation of the rank.'),
      '#options' => $normalization_options,
      '#default_value' => $this->options['normalization'],
     );
  }

  /**
   * Append the possible WHERE clause to the Views query object.
   */
  function query() {
    // Fetch the ID of the chosen filter.
    $filter_id = $this->options['filter'];
    // Capture the keyword(s) entered into the chosen filter.
    $filter_value = $this->view->postgresql_fts[$filter_id]['value'];

    // Only add the WHERE clause if filter keywords are entered.
    if($filter_value){
      // Set the FTS table which contains the lexemes.
      $fts_alias = $this->view->postgresql_fts[$filter_id]['alias'];
      // See on which column of the FTS table we need to rank.
      $fts_column = $this->view->postgresql_fts[$filter_id]['column'];
      // See which FTS query is going to be performed by the filter.
      $ts_query = $this->view->postgresql_fts[$filter_id]['formula'];

      if($this->options['cover_density']) {
        // Work out the ts_rank_cd "formula".
        $formula = 'ts_rank_cd('.$fts_alias.'.'.$fts_column.','.$ts_query.','.$this->options['normalization'].')';
      }
      else {
        // Work out the ts_rank "formula".
        $formula = 'ts_rank('.$fts_alias.'.'.$fts_column.','.$ts_query.','.$this->options['normalization'].')';
      }

      // Add the actual "ORDER BY" query.
      $this->query->add_orderby(NULL, $formula, $this->options['order']);
    }
  }

  /**
   * Tiny helper to prepare the filter listing for
   * use in a Drupal Render Array.
   */
  private function _populate_filter_dropdown($filters) {
    $filters_dropdown = array();
    // Fetch the custom FTS filter data present on the Views object
    // and build a key => value array for Drupal (key being the FTS filter ID).
    foreach($this->view->postgresql_fts as $filter) {
      $filters_dropdown[$filter['filter_id']] = t('Filter for field: !field',
                                                array('!field' => $filter['field']));
    }

    return $filters_dropdown;
  }

}
